local json = requires(IMG_PATH, "GameLuaScript/Network/dkjson")
local loading = requires(IMG_PATH, "GameLuaScript/Common/KNLoading")
local def = requires(IMG_PATH, "GameLuaScript/Scene/battleNew/def")
local ui = requires(IMG_PATH, "GameLuaScript/Scene/battleNew/ui")
local result = requires(IMG_PATH, "GameLuaScript/Scene/battle/resultLayer")
local M = class("battleNew", function() return display.newScene("battleNew") end)
M.__index = M

table.merge(M, {
	params = nil,		--外部参数
    report = nil,		--战报
    roles = {{},{}},	--角色
    backs = {},			--替补数据
    pet_sp = {},		--宠物怒气
    ui = nil,			--ui
    index = {			--播放位置
        round = 1,
        step = 1
    },
    pauseCnt = nil,		--锁定统计
    agile = {			--敏捷总值
    	self = nil,
    	target = nil
	},
    foeAgent = nil,		--对方代表人物
    speed = nil,		--速度
    isBossWar = nil,	--是否是帮派boss战斗
    rowNum = 0,			--切牌次数
    reportIdx = 1,
})

function M:create(params) return M.new(params) end
function M:ctor(params)
	-- io.writefile("/Users/kuniu/Desktop/tmpReport.txt", json.encode(DATA_Battle:get()), "w")
	self.params = params or {}
	self.reportIdx = 1
	self:start()
end

function M:start()
	local key = "report"
	if self.reportIdx > 1 then
		key = "report_" ..self.reportIdx
	end
	self.report = DATA_Battle:get()[key]
	self.roles = {{},{}}
	self.backs = {}
	self.index = {}
	self.pauseCnt = 0
	self.petSkill = {}
	self.agile = {self = 0, target = 0}
	self.foeAgent = nil
	self.speed = 1
	self.rowNum = 0
	self.boss = self.report.prepare.boss
	self.round_label = nil
	self.isBossWar = self.params and self.params._old_name == "boss"--帮派boss战
	self.ui = ui.new(self):addTo(self, display.height)
	display.newSprite(def.image .."bg" ..math.random(3) ..".jpg"):center():addTo(self)
	SimpleAudioEngine:sharedEngine():setBackgroundMusicVolume(1)
	audio.playMusic( IMG_PATH .. "sound/battle_bg.mp3" , true )
	scheduler.performWithDelayGlobal(function()--.5秒后再开始游戏
		self.lockCnt = 0
		self.index.round = 1
		self:playNextRound()
		self:scheduleUpdate(handler(self, self.update))
	end, 0)
end

function M:onExit()
	audio.stopMusic(true)

	if self.params.notes then
		for k,v in pairs(self.params.notes) do
			v:release()
		end
	end
end

function M:unline()--掉线
	self:unscheduleUpdate()
	self.roles = {{},{}}
	self:removeAllChildrenWithCleanup(true)
end

function M:showResult()
	self:lockPlay()
	if self.params.overFun then
		self.params.overFun()
		return
	end
	local showResult = function()
		self:unscheduleUpdate()
		self.roles = {{},{}}
		self:removeAllChildrenWithCleanup(true)
		self:add(result:create(), display.height*4)
	end
	if DATA_Battle:getMod() == "guide" then
		showResult()
		return
	end
	local battle_call_data = {
		report_id = DATA_Battle:get("report_id"),
		num = self.rowNum
	}
	SOCKET:getInstance("battle"):call(DATA_Battle:getMod(), DATA_Battle:getAct().. "_finish" , "finish" , battle_call_data , { success_callback = showResult } )
end

function M:playEnd()
	if DATA_Battle:get().report_count and DATA_Battle:get().report_count > self.reportIdx then
		dump(self.params)
		self:unscheduleUpdate()
		self:removeAllChildrenWithCleanup(true)
		self.reportIdx = self.reportIdx+1
		if self.params.notes then
			local node = self.params.notes[self.reportIdx]
			if node then
				node:addTo(self)
				scheduler.performWithDelayGlobal(function()
					node:removeSelf()
					self:start()
				end, 2)
				return
			end
		end
		self:start()
	else
		self:showResult()
	end
end

function M:playNextRound()
	if 4 == self.index.round then
		self:playEnd()
		return
	end
	self.index.step = 0
	self:playNextStep()
end

function M:playNextStep()
	if 0 ~= self.lockCnt then
		return
	end
	self.index.step = self.index.step + 1
	if self.index.step > #self.report[tostring(self.index.round)] then
		self.index.round = self.index.round + 1
		self:playNextRound()
		return
	end
	self:lockPlay()
	local current = self.report[tostring(self.index.round)][self.index.step]
	requires(IMG_PATH, "GameLuaScript/Scene/battleNew/step/" ..current.type, CURRENT_MODULE_NAME).launch(self, current.data, function()
		local _ = def.wait[current.type] 
		and scheduler.performWithDelayGlobal(handler(self, self.unlockPlay), def.wait[current.type]*self.speed) 
		or self:unlockPlay()
		self:updateSomething(current.data, current.type)
	end)
end

function M:updateSomething(data, type)
	if not data then
		return
	end

	--宠物怒气
	if data.p1_sp then self.ui:refreshSkill(1, data.p1_sp, type) end
	if data.p2_sp then self.ui:refreshSkill(2, data.p2_sp, type) end

	-- -- 人物血量
	-- if data.change then
	--     for i=1,2 do for j=1,4 do
	--     	local role = self.roles[i][j]
	--     	if role then
	--     		local change = comm.getChange(role, data.change)
	--     		if change then
	--     			role:handleHP(change.hp)
	--     		end
	--     	end
	--     end end	
	-- end
end

function M:lockPlay()
	self.lockCnt = self.lockCnt+1
end

function M:unlockPlay()
	self.lockCnt = self.lockCnt-1
	self:playNextStep()
end

function M:nextIsEnd()
	if self.index.step + 1 > #self.report[tostring(self.index.round)] then
		if 1 > #self.report[tostring(self.index.round+1)] then
			return true
		end
	end
	return false
end

function M:update(dt)
    for i=1,2 do for j=1,4 do
    	local role = self.roles[i][j]
    	if role and self.ui.skillUsing.mask and self.ui.skillUsing.targetIsSelf == (1==i) then--正在使用宠物技能
    		if role.lastY < display.height then
    			role.lastY = display.height*2-role.originPos.y
    			self:reorderChild(role, role.lastY)
    		end
    	elseif role and role.cutting then
		    role.lastY = display.height+2
			self:reorderChild(role, role.lastY)
		elseif role and role.attacking then
		    role.lastY = display.height-2
			self:reorderChild(role, role.lastY)
    	else
		    if role and role.lastY ~= role:getPositionY() then
		        role.lastY = role:getPositionY()
		        self:reorderChild(role, display.height-role.lastY)
		    end
    	end
    end end
end

function M:getRole(g,i)
	return self.roles[g][i+1]
end

--test for local report..
function M:test()
	local data
	local file = io.open(CCFileUtils:sharedFileUtils():fullPathFromRelativePath("temp_battle.txt"), "r")
    if file then
        local content = file:read("*a")
        io.close(file)
        data = json.decode(content)
        -- data = {report=data}
    end
	DATA_Battle:setMod("mission")
	DATA_Battle:setAct("execute")
	DATA_Battle:set( data or {} )
	DATA_IDTYPE:set({
		["1" ] = "general",
		["16"] = "prop",
		["17"] = "prop",
		["18"] = "npc",
		["19"] = "prop",
		["2" ] = "pet",
		["20"] = "prop",
		["3" ] = "skill",
		["4" ] = "petskill",
		["5" ] = "equip",
		["6" ] = "prop",
		["7" ] = "prop",
		["8" ] = "prop",
		["9" ] = "prop",
	})
	display.replaceScene(M.new())

end

return M