local def = requires(IMG_PATH, "GameLuaScript/Scene/battleNew/def")
local effect = requires(IMG_PATH, "GameLuaScript/Scene/battleNew/effect")
local btn = requires(IMG_PATH, "GameLuaScript/Common/KNBtn")
local config = requires(IMG_PATH, "GameLuaScript/Config/Petskill")
local infoLayer = requires(IMG_PATH, "GameLuaScript/Scene/battle/heroInfo")
local KNMask = requires(IMG_PATH,"GameLuaScript/Common/KNMask")
local M = class("battleUI", function() return display.newLayer() end)
M.__index = M

table.merge(M, {
	scene = nil,		--战斗场景
	backs = nil,		--替补
	skill = nil,		--幻兽技能按钮
	skillUsing = nil,	--幻兽技能使用中
	touchLayer = nil,	--触摸层
    skipLayer = nil,	--跳过层
    bottom = nil,		--底层
    top = nil,			--顶层
    petSkillTime = nil,	--宠物正在使用技能
})

function M:ctor(scene)
	self.scene = scene
    self.backs = {{}, {}}
    self.skill = {
		btn = {{}, {}},
		tip = {{}, {}},
		progress = {{}, {}},
	}
	self.skillUsing = {
    	mask = nil,
    	targetIsSelf = nil,
    	role = nil
    }

    --技能条
    if not scene.isBossWar then
		local bottom = display.newSprite(def.image .."skillBar.png"):anchor(.5, 0):pos(display.cx, 0):addTo(self)
		local top = display.newSprite(def.image .."skillBar.png"):flipY(true):anchor(.5, 1):pos(display.cx, display.height):addTo(self)

		display.newSprite(def.image .."font_petskill.png", 2, 75):anchor(0, .5):addTo(bottom)
		display.newSprite(def.image .."font_petskill.png", 2, 15):anchor(0, .5):addTo(top)

		self.bottom = bottom
		self.top = top

		for i=1,2 do
			local bar = (i==1 and bottom or top)
			for j=1,3 do
				--主动技能按钮
				local x, y = 80+(j-1)*105, 1 == i and 50 or 35
				local skillData
				if self.scene.report.prepare["p" ..i .."_pet"].skills then
				 	skillData = self.scene.report.prepare["p" ..i .."_pet"].skills["a" ..j]
				end
				local path = def.oldimage .."pet/"
				local file = "skill_fore_dis.png"
				if skillData then
					if def.debug then
						skillData.cid = 4601
					end
					path = "image/skill/"
					file = skillData.cid ..".png"
				end
				self.skill.btn[i][j] = btn:new(path, {file}, x, y, {
					front = def.oldimage .."pet/static_bg.png",
					anchMid = true,
					callback = function() 
						if i == 1 and nil ~= skillData then
							self:useSkill(skillData)
						end
					end
				})
				bar:add(self.skill.btn[i][j].layer)

				if skillData then
					--progress
					local progress = CCProgressTimer:create(display.newSprite(def.oldimage .."pet/skill_fore.png"))
				    progress:setType(kCCProgressTimerTypeRadial)
				    progress:setPosition(ccp(x, y))
					bar:add(progress)

					self.skill.progress[i][j] = progress
					self.skill.progress[i][j].cid = skillData.cid
					self.skill.progress[i][j].lv = skillData.lv

					--tip
					if 1 == i then
						self.skill.tip[i][j] = effect.get("skillTip"):pos(x, y):hide():addTo(i==1 and bottom or top)
					end
				end

				--被动技能
				local passivity
				if self.scene.report.prepare["p" ..i .."_pet"].skills then
				 	passivity = self.scene.report.prepare["p" ..i .."_pet"].skills["p" ..j]
				end
				local filename
				if passivity and passivity.cid then
					local attrib = config[passivity.cid ..""]["department"]
					filename = def.oldimage .."pet/passivity/" ..attrib ..".png"
				else
					filename = def.oldimage .."pet/passivity/kong.png"
				end
				local x, y = 29+(j-1)*105, 1 == i and 19 or 65
				display.newSprite(filename, x, y):addTo(bar)
			end
		end
	end

	--副队
	self.backs = {{}, {}}
	local font_back1 = display.newSprite(def.image .."font_backs.png", 370, 3):anchor(0, 0):addTo(self)
	local font_back2 = display.newSprite(def.image .."font_backs.png", 370, 827):anchor(0, 0):addTo(self)
	self.backs[1].label = display.strokeLabel("0", 418, 4, 18, ccc3(255, 255, 255)):addTo(self)
	self.backs[2].label = display.strokeLabel("0", 418, 827, 18, ccc3(255, 255, 255)):addTo(self)
	if scene.isBossWar then
		font_back2:hide()
		self.backs[2].label:hide()
	end

	--跳过按钮	
	local isEnabledSkip = function()
		if def.debug then
			return true
		end

		if DATA_Battle:getMod() ~= "guide" and DATA_Battle:getMod() ~= "empire" and DATA_Battle:getMod() ~= "empirearena" then 
			--副本时，vip2以上可以有跳过 commonfuben = "装备、技能、灵源、经验、淬炼 银两  副本"
			if DATA_Battle:getMod() == "commonfuben" or DATA_Battle:getMod() == "trial" then
				if DATA_Battle:getMod() == "commonfuben" and DATA_Vip:get("viplv") >= 2 then
					return true
				end
				--铁血试炼时，vip3以上可以有跳过
				if DATA_Battle:getMod() == "trial" and DATA_Vip:get("viplv") >= 3  then
					return true 			
				end
				return false
			else
				if DATA_Battle:getMod() == "mission"  then	--关卡
					local curMission = DATA_Mission:getCurMissionData()
					local starNum = DATA_Mission:get( curMission.map_id , "missions" , curMission.mission_id , "star" )
					return 0 ~= starNum
				elseif DATA_Battle:getMod() == "insequip" then	--忠义堂
					local curData = DATA_Instance:getCurEquipData()
					local maxData = DATA_Instance:get( "equip" , "current")		
					maxData.map_id = tonumber(maxData.map_id)
					maxData.ins_id = tonumber(maxData.ins_id)
					if curData.map_id == maxData.map_id then
						if  curData.ins_id > maxData.ins_id then return false end
					elseif curData.map_id > maxData.map_id then
						return false
					end
					return true
				end
				return false
			end
		end
		return true
	end

	if isEnabledSkip() then
		local showSkipBtn
		local showSkip = function()
			local layer
			layer = display.newLayer():addTo(self)
				:pos(-display.width, 0):size(display.width, display.height)
				:runs({
					CCMoveTo:create(.2, ccp(80, 0)),
					CCMoveTo:create(.1, ccp(-30, 0)),
					CCMoveTo:create(.1, ccp(0, 0)),
				})

			layer:setTouchEnabled( true )
			layer:registerScriptTouchHandler(function(event, x, y)
				showSkipBtn:setEnable(true)
				showSkipBtn:getLayer():show()
				scene:unlockPlay()
				layer:removeSelf()
			end)
			display.newSprite(def.oldimage .."skip_bg.png"):center():addTo(layer)
			local skipBtn = btn:new(def.oldimage, {"skip1.png", "skip2.png"}, display.cx, display.cy, {
				anchMid = true,
				front = def.oldimage .."skip_text.png",
				callback = function()
					scene:showResult()
				end,
			})
			layer:add(skipBtn:getLayer())
		end
		showSkipBtn = btn:new(def.oldimage, {"skip_flag.png"}, 0, 0, {
			callback = function()
				showSkipBtn:setEnable(false)
				showSkipBtn:getLayer():hide()
				scene:lockPlay()
				showSkip()
			end,
		})
		local began_x = 0
		local skip_touchLayer = display.newLayer():addTo(self):enableTouch(true):onTouch(function(eventType, x, y) 
			if eventType == CCTOUCHBEGAN then
		    	began_x = x
		    	return true
		    end
		    if eventType == CCTOUCHMOVED then
		    	return true 
		    end

		    if eventType == CCTOUCHENDED then
		    	if x - began_x > 100 then 
		    		showSkipBtn:setEnable(false)
					showSkipBtn:getLayer():hide()
					scene:lockPlay()
					showSkip()
		    	end

		        return true
		    end
			return false
		end , false, -140, false)




		CCNodeExtend.extend(showSkipBtn.layer):anchor(0, .5):pos(-10, display.cy-50):addTo(self)
	end

	--加速播放
	local fu = display.newSprite(def.image .."num3-.png", display.width-70, display.cy+65):addTo(self):hide()
	local speedBtn, speedUpt
	speedBtn = btn:new(def.image, {"x1.png"}, 0,0, {
		anchMid = true,
		callback = function() 
			if scene.speed == 1 then
				scene.speed = .5
			elseif scene.speed == .5 then
				scene.speed = .25
			-- elseif scene.speed == .25 then
			-- 	scene.speed = 2
			-- elseif scene.speed == 2 then
			-- 	scene.speed = 4
			else
				scene.speed = 1
			end
			_g_battle_speed = scene.speed
			speedUpt()
		end,
	})
	CCNodeExtend.extend(speedBtn.layer):pos(display.width-30, display.cy+70):addTo(self)

	speedUpt = function()
		if scene.speed == 1 then
			fu:hide()
			speedBtn:setBg(1, def.image .."x1.png")
		elseif scene.speed == .5 then
			fu:hide()
			speedBtn:setBg(1, def.image .."x2.png")
		elseif scene.speed == .25 then
			fu:hide()
			speedBtn:setBg(1, def.image .."x4.png")
		elseif scene.speed == 2 then
			fu:show()
			speedBtn:setBg(1, def.image .."x2.png")
		else
			fu:show()
			speedBtn:setBg(1, def.image .."x4.png")
		end
	end

	_g_battle_speed = _g_battle_speed or scene.speed
	scene.speed = _g_battle_speed
	speedUpt()

	--touch layer
	self.touchLayer = display.newLayer():addTo(self):enableTouch(true):onTouch(function(event, x, y) 
		if CCTOUCHENDED == event then
			for i=1,2 do for j=1,4 do
		    	local role = self.scene.roles[i][j]
				if role and role.hp.cur > 0 and role:getTouchRect():containsPoint(ccp(x, y)) then
					infoLayer:new(role.data, {
						scene = self.scene, 
						bufs = role.bufs, 
						hp = role.hp.cur
					})
					return
				end
		    end end
		end
		return true
	end)
end

function M:initBacks(group, data)
	self.backs[group].label:setString(#data .."")
	local cnt = 0
	local posx=35
	local posy=40
	if "tgxy2"==CHANNEL_GROUP or "tgxy"==CHANNEL_GROUP then 
		posx=33
		posy=33
	end 

	for i=4, 1, -1 do
		local v = data[i]
		if v then
			cnt = cnt+1
			local hero_img_path = IMG_PATH .. "image/hero/"
			if HAS_GOD_GENERAL and v["god"] == 1 then
				hero_img_path = IMG_PATH .. "image/hero/shen/"-- 天神
			end
			local spr = display.newSprite(def.oldimage .."backFrame.png"):add(display.newSprite(hero_img_path ..getImageByType( v.cid or v.npc_id , "s" , true, group )):pos(posx,posy))
			if "1" == tostring(group) then
				spr:addTo(self):pos(410-cnt*5, 63)
			else
				spr:addTo(self):pos(410-cnt*5, 788)
			end
			self.backs[group]["" ..(v._index+1)] = spr
		end
	end
end

function M:removeBack(group, index)
	if self.backs[group]["" ..index] then
		self.backs[group]["" ..index]:removeSelf()
		self.backs[group]["" ..index] = nil
	end

	local cnt = 0
	for i=4, 1, -1 do
		local spr = self.backs[group]["" ..i]
		if spr then
			cnt = cnt+1
			if "1" == tostring(group) then
				spr:pos(410-cnt*5, 63)
			else
				spr:pos(410-cnt*5, 788)
			end
		end
	end
	self.backs[group].label:setString(cnt .."")
end

function M:refreshSkill(group, sp, type)
	self.scene.pet_sp[group] = sp

	if self.scene.isBossWar then
		return
	end

	if self.petSkillTime then
		if 1 == group and type == "skillpetnew" then
			self.petSkillTime = false
		end
		return
	end

	for i,v in ipairs(self.skill.progress[group]) do
		local max
		if not config["" ..v.cid] then
			max  = 100
		else
			max = config["" ..v.cid]["" ..v.lv].max_sp
			max = max == 0 and 1 or max
		end
		local progress = math.min(100, sp/max*100)
		if v.last ~= progress then
			if 100 == v.last and self.skill.tip[group][i] then
				self.skill.tip[group][i]:hide()
			end
			v.last = progress
			v:runAction(CCProgressTo:create(.3, v.last))
			if 100 == progress and self.skill.tip[group][i] then
				self.skill.tip[group][i]:show()
			end
		end
	end
end

function M:useSkill(skill)
	if self.scene.isBossWar then
		return
	end

	if def.isGodwarBattle(1) and def.isGodwarBattle(2) then
		return
	end

	if DATA_Battle:getMod() == "guide" or DATA_Battle:getMod() == "empirearena" then
		print("can't use skill")
		return
	end

	if self.skillUsing.mask then
		return
	end

	if self:getParent():nextIsEnd() then
		return
	end

	if not config["" ..skill.cid] then
		CCLuaLog("宠物技能[" ..skill.cid .."]不存在")
		return
	end

	local data = config["" ..skill.cid]
	if self.scene.pet_sp[1] < data["" ..skill.lv].max_sp then
		return
	end

	local useSkillEnd = function(isSuccess, err)
		if not isSuccess then
			KNMsg.getInstance():flashShow("[" .. err.code .. "]" .. err.msg)
		else
			self.scene.report = DATA_Battle:get().report
		end
		self.skillUsing.role:skillUnselected()
		self.skillUsing.mask:removeSelf()
		self.skillUsing = {}
		self.touchLayer:enableTouch(true)
		self.scene:unlockPlay()
	end

	local sendMsg = function(role)
		local battle_call_data = {
			report_id = DATA_Battle:get("report_id"),
			turn = self.scene.index.round,
			step = math.min(#self.scene.report[tostring(self.scene.index.round)], self.scene.index.step)-1,
			pet_skill_id = skill.cid,
			target_id = role.data.id or "",
			index = role.data._index or ""
		}
		SOCKET:getInstance("battle"):call(DATA_Battle:getMod() , DATA_Battle:getAct() .. "_process" , "process" , battle_call_data , {
			error_callback = function(err) useSkillEnd(false, err) end,
			success_callback = function() useSkillEnd(true) end
		})
		self:refreshSkill(1, 0)
		self.petSkillTime = true
	end

	local useSkill = function()
		self.touchLayer:enableTouch(false)
		self.scene:lockPlay()
		self.skillUsing.targetIsSelf = 5 == tonumber(data.type) or 5 == tonumber(skill.type)
		self.skillUsing.mask = CCLayerExtend.extend(CCLayerColor:create(ccc4(0, 0, 0, 188), display.width, display.height))
			:addTo(self)
			:enableTouch(true)
			:onTouch(function(event, pos) 
				if CCTOUCHENDED == event then
					for i,v in ipairs(self.scene.roles[self.skillUsing.targetIsSelf and 1 or 2]) do
						if v:getTouchRect():containsPoint(ccp(pos[1], pos[2])) then
							self.skillUsing.role = v
							v:skillSelected()
							sendMsg(v)
							return
						end
					end
				end
			end)
		display.newSprite(def.image .."select_title.png")
			:pos(display.cx, display.cy*(self.skillUsing.targetIsSelf and .9 or 1.9))
			:addTo(self.skillUsing.mask)
			:run(CCRepeatForever:create(tolua.cast(CCSequence:createWithTwoActions(CCScaleTo:create(1, .5), CCScaleTo:create(1, 1)), "CCActionInterval")))
	end

	useSkill()
end


return M