
------------------------------------------------
-- common head
local modname = ...
local M = {}
setmetatable( M, { __index = _G } )
package.loaded[modname] = M
setfenv(1, M)
------------------------------------------------


local KNBtn = requires(IMG_PATH , "GameLuaScript/Common/KNBtn")
local KNInputText = requires(IMG_PATH, "GameLuaScript/Common/KNInputText")

local layer

------------- 公共代码
local tg_common_login = function(self, callback)
	layer = display.newLayer()

	requires(IMG_PATH, "GameLuaScript/Config/version")

	local bg = display.newSprite(IMG_PATH .. "image/scene/login/bg.jpg")
	setAnchPos(bg, 0, 0)
	layer:addChild(bg)

	local version_label = CCLabelTTF:create(VERSION, FONT, 18)
	setAnchPos(version_label, 10, 10)
	version_label:setColor( ccc3( 0x4d, 0x15, 0x15 ) )
	layer:addChild(version_label)
	
	local loginBtn
	loginBtn = KNBtn:new(IMG_PATH .. "image/scene/login", {"select_btn.png", "select_btn_pre.png"} , display.cx - 100, 70,
	{
		front = IMG_PATH .. "image/scene/login/login.png", 
		callback = function()
			callback()
		end
	})
	layer:addChild( loginBtn:getLayer() )
	
	return layer, loginBtn
end

local parse_response = function(str, sep1, sep2)
	local response = string.split(str, sep1)
	
	local data = {}

	for i = 1, #response do
		local sep = string.find(response[i], sep2, 1, true)
		if sep ~= nil then
			local k = string.sub(response[i], 1, sep - 1)
			local v = string.sub(response[i], sep + 1, -1)
			
			if k ~= nil and k ~= "" then
				data[k] = v and v or ""
			end
		end
	end
	
	dump(data, "response")
	
	return data
end

local login = function(self, params)
	-- 参数默认值
	local loginParameters = params.loginParameters or {}	-- table
	local onLoginSucceed = params.onLoginSucceed or function()	-- function expected
		KNMsg.getInstance():flashShow("no onLoginSucceed action!")
	end
	-------------------------------------------------------------------
	local layer, loginBtn
	layer, loginBtn = tg_common_login(self, function()
	-------------------------------------------------------------------
		local onLoginBegin = function()
			loginBtn:setEnable(false)
			
			-- 1秒内禁止重复点击
			local array = CCArray:create()
			array:addObject( CCDelayTime:create(1) )
			array:addObject( CCCallFunc:create( function() loginBtn:setEnable(true) end ) )
			loginBtn:getLayer():runAction( CCSequence:create(array) )
		end
		
		local onLoginEnd = function()
			
		end
		
		onLoginBegin()
		
		loginParameters.action = "login"
		loginParameters.callback = function(response)
			CCLuaLog("====================>lua login<========================")
			dump(response, "response")
		
			if response["result"] == "fail" or response["result"] == "cancel" then
				if response["message"] and response["message"] ~= "" then
					KNMsg.getInstance():flashShow(response["message"])
				end
				return
			elseif response["result"] == "succeed" then
				onLoginSucceed(response)
			else
				CCLuaLog("do nothing")
			end
		end
		
		DirectChannel.instance():call(loginParameters)
	-------------------------------------------------------------------
	end)
	
	return layer
end

local login_template = function(self, params, ...)
	-- local vararg = {...}
	-- CCLuaLog("vararg params=")
	-- for i, v in pairs(vararg) do
		-- CCLuaLog( tostring(v) )
	-- end
	
	local post = params.post_data
	local wait = params.wait
	local login = params.login
	local thiz = params.thiz
	local success_callback = params.success_callback or function() end
	-------------------------------------------------------------------
	local layer, loginBtn
	layer, loginBtn = tg_common_login(self, function()
	-------------------------------------------------------------------
		local KNLoading, loading
		
		local onLoginBegin = function()
			if wait then
				KNLoading = requires(IMG_PATH,"GameLuaScript/Common/KNLoading")
				loading = KNLoading:new()
				local scene = display.getRunningScene()
				scene:addChild( loading:getLayer() )
			else
				loginBtn:setEnable(false)
				
				-- 1秒内禁止重复点击
				local array = CCArray:create()
				array:addObject( CCDelayTime:create(1) )
				array:addObject( CCCallFunc:create( function() loginBtn:setEnable(true) end ) )
				loginBtn:getLayer():runAction( CCSequence:create(array) )
			end
		end
		
		local onLoginEnd = function()
			if wait then
				loading:remove()
			else
			end
		end
		
		onLoginBegin()
		login(thiz, function(response)
			CCLuaLog("====================>lua login<========================")
			local data = parse_response(response["data"], "@@", "=")
			
			if data["result"] == "fail" or data["result"] == "cancel" then
				if data["message"] and data["message"] ~= "" then
					KNMsg:getInstance():flashShow(data["message"])
				end
				onLoginEnd()
				return
			elseif data["result"] == "succeed" then
				-- 准备数据
				-- 每次构造一份新的数据,
				-- 以防第一次点击登录按钮失败后第二次点击出现问题
				local post_data = {}
				for k, v in pairs(post) do
					post_data[k] = v
				end
				
				dump(post_data, "post")
				for k, v in pairs(post_data) do
					
					if k ~= "platform" then
						if type(v) == "table" then
							post_data[k] = v[2](data[v[1]])
						else
							post_data[k] = data[v]
						end
					end
				end
				
				post_data.channel = CHANNEL_ID
				for k , v in pairs(device.infos) do
					post_data[k] = v
				end
				
				dump(post_data, "post_data")
				
				onLoginEnd()
				
				-- 发请求
				HTTP:call("verify" , "check" , post_data,
				{
					success_callback = success_callback
				})
			elseif data["result"] == "other" then
				onLoginEnd()
				CCLuaLog("login result=other, do nothing")
			end
		end)
	-------------------------------------------------------------------
	end)
	
	return layer
end

tgxydl_login = function(self, params)
	return login_template( self,
	{
		post_data =
		{
			platform = "tgxy_dl",
			mid = "mid",
			token = "token"
		},
		wait = false,
		login = dlSDK:getInstance().dlLogin,
		thiz = dlSDK:getInstance(),
	})
end

tgxyqihoo_login = function(self, params)
	return login_template( self,
	{
		post_data =
		{
			platform = "tgxy_360",
			qihoo_code = "code"
		},
		wait = false,
		login = san60SDK:getInstance().san60Login,
		thiz = san60SDK:getInstance(),
		success_callback = function(data)
			dump(data["result"], "login succeed")
			local data = data["result"]["hy_platform_info"]
			
			local handle
			handle = CCDirector:sharedDirector():getScheduler():scheduleScriptFunc(function()
				CCDirector:sharedDirector():getScheduler():unscheduleScriptEntry(handle)
				handle = nil
				san60SDK:getInstance():san60Query(data["token"]["access_token"], data["id"])
			end, 10, false)
		end
	})
end

tgxyDK_login = function(self, params)
	layer = display.newLayer()

	requires(IMG_PATH,"GameLuaScript/Config/version")
	local KNLoading = requires(IMG_PATH,"GameLuaScript/Common/KNLoading")

	
	local bg = display.newSprite(IMG_PATH .. "image/scene/login/bg.jpg")
	setAnchPos(bg , 0 , 0)
	layer:addChild( bg )

	local version_label = CCLabelTTF:create(VERSION , FONT , 18)
	setAnchPos(version_label , 10 , 10)
	version_label:setColor( ccc3( 0x4d , 0x15 , 0x15 ) )
	layer:addChild( version_label )
	
	local loginBtn
	loginBtn = KNBtn:new(IMG_PATH .. "image/scene/login", {"select_btn.png" , "select_btn_pre.png"} , display.cx - 100 , 70 , {
		front = IMG_PATH .. "image/scene/login/login.png" , 
		callback = function()
			local loading = KNLoading:new()
			local scene = display.getRunningScene()
			scene:addChild( loading:getLayer() )
			
			hyDKSDK:getInstance():hyDKLogin(function(response)
				response = string.split(response["data"] , "@@")
				local data = {}

				for i = 1 , #response do
					local temp = string.split(response[i] , "=")
					if #temp == 2 then
						data[temp[1]] = temp[2]
					end
				end

				if data["result"] == "fail" or data["result"] == "cancel" then
					KNMsg:getInstance():flashShow(data["message"])
					loading:remove()
					return
				end

				CCLuaLog("uid : " .. (data["uid"] or "nil"))
				CCLuaLog("sessionid : " .. (data["sessionid"] or "nil"))
				
				-- 发请求
				local post_data =
				{
					platform = "tgxy_dk",
					uid = data["uid"] ,
					sessionid = data["sessionid"],
					channel = CHANNEL_ID,
				}
				for k , v in pairs(device.infos) do
					post_data[k] = v
				end
				
				loading:remove()
				HTTP:call("verify" , "check" , post_data)
				
			end, function()
				Clock:removeTimeFun("heartbeat")
				SOCKET:getInstance("battle"):close()
				SOCKET:delInstance("battle")
				switchScene("login")
			end)
		end
	}):getLayer()
	layer:addChild(loginBtn)
	
	return layer
end

tgxyuc_login = function(self, params)
	return login_template( self,
	{
		post_data =
		{
			platform = "tgxy_uc",
			uc_sid = "sid"
		},
		wait = false,
		login = ucSDK:getInstance().ucLogin,
		thiz = ucSDK:getInstance(),
	})
end

dhshuc_login = function(self, params)
	return login_template( self,
	{
		post_data =
		{
			platform = "uc",
			uc_sid = "sid"
		},
		wait = false,
		login = ucSDK:getInstance().ucLogin,
		thiz = ucSDK:getInstance(),
	})
end


tgxymi_login = function(self, params)
	return login(self,
	{
		loginParameters = { params1 = "one", params2 = "two", params3 = "three", },
		onLoginSucceed = function(response)
			local post_data =
			{
				platform = "tgxy_mi",
				mi_uid = response["uid"],
				mi_session = response["session"],
				----------------------------
				channel = CHANNEL_ID,
			}
			for k , v in pairs(device.infos) do
				post_data[k] = v
			end
			
			HTTP:call("verify" , "check" , post_data)
		end
	})
end


local channel_tg =
{
	--------------------------太古------------------------------
	tgxydl = tgxydl_login,
	tgxyqihoo = tgxyqihoo_login,
	tgxyDK = tgxyDK_login,
	["tgxy.uc"] = tgxyuc_login,
	["tgxy.mi"] = tgxymi_login,


	["uc"] = dhshuc_login,

}

go = function(self, params)
	local capture = channel_tg[CHANNEL_ID]
	if capture then
		capture(self, params)
	else
		layer = nil
	end
	
	return capture, layer
end

------------------------------------------------
--
return M
------------------------------------------------