local PATH = IMG_PATH.."image/scene/war/"
local HOLD, MOVE, DIE, HIDE = 0, 1, 2, 3
local Role = {
	layer,
	moving,
	stateIcon,
	state,
	data,
	name,
	nameBg,
	time ,   --操作 的时间
	restTime,  --移动时间
	moveLine,
	isUser,     --是否是玩家自己
	buff,  --状态加成
	die
	
}

function Role:new(x, y,state, params)
	local this = {}
	self.__index = self
	setmetatable(this, self)
	
	this.stateIcon = {}
	this.data = params or {}
	
	this.layer = display.newLayer()
	this.layer:ignoreAnchorPointForPosition(false)
	setAnchPos(this.layer, x, y, 0.5, 0.5)
	
	if tonumber(this.data.info.uid) == tonumber(DATA_User:get("uid")) then
		this.isUser = true
	end

	if this.isUser then
		this.stateIcon[MOVE] = display.newSprite(PATH.."sign_self.png")		
	else
		this.stateIcon[MOVE] = display.newSprite(PATH.."sign_our.png")		
	end
	this.stateIcon[HOLD] = display.newSprite(PATH.."flag_our.png")		

	
	this.stateIcon[MOVE]:setVisible(false)
	this.stateIcon[HOLD]:setVisible(false)
	
	this.layer:setContentSize(this.stateIcon[MOVE]:getContentSize())
	
	setAnchPos(this.stateIcon[MOVE])
	setAnchPos(this.stateIcon[HOLD], this.stateIcon[HOLD]:getContentSize().width / 4, this.stateIcon[HOLD]:getContentSize().height / 4)
	
	this.layer:addChild(this.stateIcon[MOVE])
	this.layer:addChild(this.stateIcon[HOLD], 1)
	
	
	this.nameBg = display.newSprite(PATH.."name_bg.png")
	setAnchPos(this.nameBg,this.layer:getContentSize().width / 2, -25, 0.5)
	this.layer:addChild(this.nameBg)
	
	if this.isUser then
--		this.moveLine = display.newSprite(PATH.."move_line.png")
--		setAnchPos(this.moveLine, this.layer:getContentSize().width / 2 - this.moveLine:getContentSize().width / 2, 0, 0, 1)
--		this.layer:addChild(this.moveLine)
		
		this.restTime = this.data.time
		this.name =  createLabel({str = timeConvert(this.restTime, nil, true), size = 18, color = ccc3(0xe2, 0xe2, 0xe2)})		
		Clock:addTimeFun("war_role", function()
			if this.state == MOVE then
				if this.restTime > 0 then
					this.restTime = this.restTime - 1
					this.name:setString(timeConvert(this.restTime, nil, true))
				end
			else
				this.restTime = this.data.time
			end
		end)
	else
		this.name = createLabel({str = this.data.info.name, size = 18, color = ccc3(0xe2, 0xe2, 0xe2), width = 200})
	end
	setAnchPos(this.name, this.layer:getContentSize().width / 2, -12, 0.5, 0.5)
	this.layer:addChild(this.name)
	
	if state then
		this:setState(state)
		if state == MOVE then
			this.moving = true
		end
	else
		this:setState(HIDE)
	end
	
	if params.info.effect then
		local name = params.info.effect.name
		if name == "stop" or name == "jump" then
			this.buff = display.newSprite(PATH..name..".png")
			setAnchPos(this.buff, this.layer:getContentSize().width)
			this.buff:setScale(0.3)
			this.layer:addChild(this.buff)
		end
	end
	
	return this
end

function Role:setState(state, noTime)
	self.time = os.time() + TIME_OFFSET
	
	if state == DIE then
		self.die = true
	end
	
	if self.stateIcon[self.state] then
		self.stateIcon[self.state]:setVisible(false)
	end
	
	self.state = state
	
	if state == HOLD or state == HIDE or state == DIE then
		self.moving = false
		self.layer:setRotation(0)
		if self.isUser then
			self.name:setVisible(false)
			self.nameBg:setVisible(false)
		end
	else
		self.name:setVisible(true)
		self.nameBg:setVisible(true)
	end
	
	if self.stateIcon[state] then
		self.stateIcon[state]:setVisible(true)
	end
	
	if noTime then
		self.restTime = noTime
	end
end

function Role:hold(x, y, hide, info)
--	if hide then
		self:setState(HIDE)
--	else
--		self:setState(HOLD)
--	end
	self.data.info = info
	self.layer:stopAllActions()
	self.layer:setPosition(ccp(x, y ))
	
	if self.buff then
		self.layer:removeChild(self.buff, true)
		self.buff = nil
	end	
	
	if info.effect then
		local name = info.effect.name
		if name == "stop" or name == "jump" then
			self.buff = display.newSprite(PATH..name..".png")
			setAnchPos(self.buff, self.layer:getContentSize().width)
			self.buff:setScale(0.3)
			self.layer:addChild(self.buff)
		end
	end
end

function Role:battle(x, y)
	self:setState(HIDE)
	self.layer:stopAllActions()
	self.layer:setPosition(ccp(x, y))
end

function Role:backHome(desX, desY, info)
	if self.buff then
		self.layer:removeChild(self.buff, true)
	end
	self.die = true
	self.data.info = info
	self:setState(HOLD)	
	self.layer:stopAllActions()
	self.layer:runAction(getSequenceAction(CCMoveTo:create(1, ccp(desX, desY)), CCCallFunc:create(function()
		self:setState(HIDE)
	end)))
	
end

function Role:move(x, y, info)
	local action_time = self.data.time
	
	if info and info.effect then
		local name = info.effect.name
		if name == "fast" or name == "slow" then
			action_time = info.effect.speed or (name == "fast" and self.data.time - 10 or self.data.time + 10)
		elseif name == "stop" then
			KNMsg.getInstance():flashShow("你被禁足了，休息一会吧*_*")
			return
		elseif name == "jump" then
			KNMsg.getInstance():flashShow("准备跳跃,将瞬间到达目标")
			return
		end
		
		if not self.buff then
			self.buff = display.newSprite(PATH..name..".png")
			setAnchPos(self.buff, self.layer:getContentSize().width)
			self.buff:setScale(0.3)
			self.layer:addChild(self.buff)
		end
	end
	self.restTime = action_time
	
	if self.isUser then
		self.name:setString(timeConvert(self.restTime, nil , true))
	end
--	
--	if self.moving then
--		print(os.time(), TIME_OFFSET)
--		print(info.time)
--		
		if os.time() + TIME_OFFSET - info.time > 0 then
			print("时间有误差，较正")
			self.layer:stopAllActions()
			action_time = action_time - (os.time() + TIME_OFFSET - info.time)
			self.restTime = action_time		
--		else
--			return 
		end
--	end
	
	local curX = self.layer:getPositionX()
	local curY = self.layer:getPositionY()
	
	local dx = x - curX
	local dy = y - curY 
	
	local distance = math.sqrt(math.pow(dx, 2) + math.pow(dy, 2))
	
	local degree = -math.deg(math.acos(dx / distance)) - 90
	if dy < 0 then
		degree = -degree
	elseif dy > 0 then
		degree = degree + 180
	elseif dy == 0 and dx < 0 then
		degree = 180
	end
--	if y < curY then
--		if x < curX then
--			degree = degree - 90
--		elseif x == curX then
--			degree = -180
--			self.name:setRotation(180)
--		else
--			degree = degree + 90
--		end
--	else
--		self.name:setRotation(0)
--	end


	if info then
		self.data.info = info
	end
	
	self.moving = true
	self:setState(MOVE)
	self.layer:runAction(getSequenceAction(CCRotateTo:create(0, degree),
		CCMoveTo:create(action_time, ccp(x, y))))
end

function Role:isMoving()
	return self.moving
end


function Role:getPos()
	return self.data.info.data
end

function Role:getInfo()
	return self.data.info
end

function Role:isBattle()
	return self.data.info.stat == "battle"
end

function Role:isUserSelf()
	return self.isUser
end

function Role:isDie()
	return self.die
end


function Role:getLayer()
	return self.layer
end

function Role:getRestTime()
	return self.restTime
end

return Role
