
local ColorText = {}

function ColorText:create( x, y , width , height , params )	
	local this = {}
	setmetatable( this , self )
	self.__index = self

	this.params = params or {}

	this.x = x or 0  
	this.y = y or 0 

	this.width = width 
	this.height = height or 0
	this.fontSize = this.params.fontSize or 16
	
	this.layer = display.newLayer()

	this.items = {}
	this.str = ""
	
	if this.params.debug then 
		this.layer = CCLayerColor:create(ccc4(255 , 0 , 0 , 100))
	end

	if this.params.anchorPoint then 
		this.layer:ignoreAnchorPointForPosition(false)
		this.layer:setAnchorPoint( ccp(this.params.anchorPoint.x or 0 , this.params.anchorPoint.y or 0) )
	end

	return this
end


function ColorText:initRender()

	local lineCount = self:getLineCount() 

	local renderHeight = lineCount * ( (self.fontSize + 1) + self.params.paddingTop or 0)

	if renderHeight > self.height then 
		self.height = renderHeight
	end

	self.layer:setContentSize( CCSize( self.width , self.height ) )
	self.layer:setPosition( ccp( self.x or 0,self.y or 0 ) )

	self.point = ccp(0,self.height)

	self.render = CCRenderTexture:create( self.width , self.height )
	self.render:clear(0.0 , 0.0 , 0.0 , 0.0)
	self.render:setAnchorPoint( ccp(0,0) )
	self.render:getSprite():setAnchorPoint(ccp(0,1))

	self.layer:addChild( self.render )
	
end

function ColorText:getLineCount()
	local width , lineCount = 0 , 1

	for i = 1 ,  table.nums(self.items) do
		local item = self.items[i]
		local str = tostring( item.str ) or ""
		local pos = 1

		while pos <= str:len() do
		    local n = string.byte(str, pos)
		    local l = self:getBitLenByAnsi(n)

		    local wordWidth =  self:getStrWidth( str:sub(pos , pos+l-1) , self.fontSize )

		    pos = pos + l
		    width = width + wordWidth
		    if wordWidth > self.width - width then
		        width = 0
		        lineCount = lineCount + 1
		    end
		 
		end
	end
	return lineCount
end


function ColorText:getBitLenByAnsi( n )

	local len = 2

	if n<0xc0 then                       	 
    	len = 1
	elseif n<0xe0 then                  	 
		len = 2
	elseif n<0xf0 then                 		
		len = 3
	elseif n<0xf8 then               		
		len = 4
	elseif n<0xfc then               		
		len = 5
	else                                  	
		len = 6
	end

	return len
end

function ColorText:getStrWidth(str , fontSize)

	if str == nil then 
		return 0
	end

	return CCLabelTTF:create(str,FONT,fontSize):getContentSize().width
end


function ColorText:addString(s,fColor)

	self.str = self.str .. (s or "")
	table.insert( self.items , { str = s , color = fColor } )

end

function ColorText:draw()

	self:initRender()
	local point = self.point

	for i = 1 , table.nums(self.items) do

		local item = self.items[i]
		local str = tostring( item.str ) or ""
		local pos = 1

		while pos <= str:len() do
		    
		    local n = string.byte(str, pos)
		    local l = self:getBitLenByAnsi(n)

		    local label = CCLabelTTF:create(str:sub(pos , pos+l-1),FONT,self.fontSize)
		    label:setColor(item.color or ccc3(255,232,234))
		    label:setAnchorPoint( ccp(0,1) )

		    local width = label:getContentSize().width

		    self.render:begin()
		    label:setPosition( point )
		    label:visit()
		    self.render:endToLua()

		    pos = pos + l
		    point.x = point.x + width
		    if point.x > self.width - width then
		        point.x = 0
		        point.y = point.y - self.fontSize - (self.params.paddingTop or 0)
		    end
		 
		end

		self.point = point
	end

end

function ColorText:getStr()
	return self.str
end

function ColorText:getLayer()
	self:draw()
	return self.layer
end

function ColorText:getHeight()
	return self.height 
end

return ColorText