local PATH = IMG_PATH.."image/scene/war/"
local M ={
	layer,
	info,
	camp,  --所属阵营
	icon,
	restTime,  --移动计时
	maps,  --地图
	tempLayer,
	moveTime,
	name,
	sex,
	recoveryTime, --复活次数
	tip  --当前所在位置
}

function M:new(data, camp,maps, time)
	local this = {}
	setmetatable(this,self)
	self.__index = self
	
	self.maps = maps
	self.moveTime = time
	
	this.name = data.name
	this.sex = data.sex
	this.lv = data.lv
	this.recoveryTime = data.remainRevive
	this.layer = display.newLayer()
	this.layer:ignoreAnchorPointForPosition(false)
	
	this.camp = camp
	this.restTime = 10
	
	this:update(data)
	return this
end

function M:getLayer()
	return self.layer
end


function M:getTimes()
	return self.recoveryTime
end


function M:subTimes()
	self.recoveryTime = self.recoveryTime - 1
end

function M:update(data)
	self.info = data
	self:setState()
	if self.info.stat == 0  then
		self:backHome()	
	elseif self.info.stat == 1 then
		self:hold(self.maps[self.info.x][1], self.maps[self.info.x][2])
	elseif self.info.stat == 2 then
		self:hold(self.maps[self.info.x][1] + (self.maps[self.info.y][1] - self.maps[self.info.x][1]) * (self.info.z / self.moveTime), self.maps[self.info.x][2] + (self.maps[self.info.y][2] - self.maps[self.info.x][2]) * (self.info.z / self.moveTime))
		self:move(self.maps[self.info.y][1], self.maps[self.info.y][2])
	elseif self.info.stat == 3 then
		self:battle()
	end
	
	if self.tip and self.info.stat ~= 1 then
		self.layer:removeChild(self.tip, true)
		self.tip = nil
	end
end

function M:backHome()
	if self.info.x ~= (self.camp == "serverA" and 1 or #self.maps) then
		self.layer:runAction(getSequenceAction(CCMoveTo:create(1, self.camp == "serverA" and ccp(self.maps[1][1], self.maps[1][2]) or ccp(self.maps[#self.maps][1], self.maps[#self.maps][2])), CCCallFunc:create(function()
			if self.icon then
				self.layer:removeChild(self.icon, true)
			end
			
			if self.tempLayer then
				self.layer:removeChild(self.tempLayer, true)	
			end
		end)))
	else
		if self.icon then
		self.layer:removeChild(self.icon, true)
		end
		
		if self.tempLayer then
			self.layer:removeChild(self.tempLayer, true)	
		end
	end
end

function M:hold(x, y)
	setAnchPos(self.layer, x, y, 0.5, 0.5)
	self.layer:setRotation(0)
	
	if self.info.uid == tonumber(DATA_User:get("uid")) and not self.tip then
		local frames = display.newFramesWithImage(IMG_PATH.."image/scene/multiwar/cur.png" , 11 )
		self.tip = display.playFrames(x, y,frames, 0.2, {
			forever = true,
		})
		setAnchPos(self.tip, self.layer:getContentSize().width / 2, self.layer:getContentSize().height / 2 + 30, 0.5, 0.5)
		self.layer:addChild(self.tip,2 )	
	end
end

function M:battle()
	self.layer:stopAllActions()
	setAnchPos(self.tempLayer, self.icon:getContentSize().width / 2, self.icon:getContentSize().height / 2)
	self.layer:setRotation(0)
end

function M:move(x, y)

	local curX = self.layer:getPositionX()
	local curY = self.layer:getPositionY()
	
	local dx = x - curX
	local dy = y - curY 
	
	local distance = math.sqrt(math.pow(dx, 2) + math.pow(dy, 2))
	
	local degree = -math.deg(math.acos(dx / distance)) + 90

	if y < curY then
		if x < curX then
			degree = degree - 90
		elseif x == curX then
			degree = -180
		else
			degree = degree + 90
		end
	end
	
	self.layer:setRotation(degree)
	
	self.layer:runAction(getSequenceAction(CCMoveTo:create(self.restTime - 0.5, ccp(x, y)), CCCallFunc:create(function()
		print("move done")
	end)))
end

function M:getCamp()
	return self.camp
end

function M:setState()
	if self.icon then
		self.layer:removeChild(self.icon, true)
	end
	
	if self.tempLayer then
		self.layer:removeChild(self.tempLayer, true)	
	end
	
	self.tempLayer = display.newLayer()
	self.tempLayer:ignoreAnchorPointForPosition(false)
	setAnchPos(self.tempLayer)
	
	self.layer:addChild(self.tempLayer)
	
	if self.info.stat == 1 or self.info.stat == 0 then -- 等待
		if self.camp == "serverA" then
			self.icon = display.newSprite(PATH.."flag_enemy.png")
		else
			self.icon = display.newSprite(PATH.."flag_our.png")
		end
		
		if self.info.uid == tonumber(DATA_User:get("uid")) then
			Clock:removeTimeFun("swar_move_time")
		end
		setAnchPos(self.icon, self.icon:getContentSize().width / 2 + 13, self.icon:getContentSize().height / 2 + 25, 0.5, 0.5)
	elseif self.info.stat == 2 then --移动
		self.restTime = self.moveTime - self.info.z	
		
		if self.info.uid == tonumber(DATA_User:get("uid")) then
			self.icon = display.newSprite(PATH.."sign_self.png")
		elseif self.camp == "serverA" then
			self.icon = display.newSprite(PATH.."sign_enemy.png")
		elseif self.camp == "serverB" then
			self.icon = display.newSprite(PATH.."sign_our.png")
		end
		print("#########################"..self.camp)
	
		setAnchPos(self.icon, self.icon:getContentSize().width / 2, self.icon:getContentSize().height / 2, 0.5, 0.5)
	
		local nameBg = display.newSprite(PATH.."name_bg.png")
		setAnchPos(nameBg,self.icon:getContentSize().width / 2, -25, 0.5)
		self.tempLayer:addChild(nameBg)
		
		local name 
		if self.info.uid ==tonumber(DATA_User:get("uid")) then
			name = createLabel({str = timeConvert(self.restTime, nil, true), size = 18, color = ccc3(0xe2, 0xe2, 0xe2), width = 200})
			Clock:addTimeFun("swar_move_time", function()
				if self.restTime > 0 then
					self.restTime = self.restTime - 1
					name:setString(timeConvert(self.restTime, nil, true))
				end
			end)
		else
			name = createLabel({str = self.name, size = 18, color = ccc3(0xe2, 0xe2, 0xe2), width = 200})
		end
		setAnchPos(name, self.icon:getContentSize().width / 2, -12, 0.5, 0.5)
		self.tempLayer:addChild(name)
	elseif self.info.stat == 3 then  --战斗
		local frames = display.newFramesWithImage(IMG_PATH .. "image/scene/war/fight_effect.png" , 6 )
		self.icon = display.playFrames(0, 0, frames, 0.1, {
			forever = true,
		})
	end
			
	self.layer:setContentSize(self.icon:getContentSize())
	self.tempLayer:setContentSize(self.icon:getContentSize())

	
	
	self.tempLayer:addChild(self.icon)
end

function M:getSex()
	return self.sex
end

function M:getLv()
	return self.lv
end

function M:getName()
	return self.name
end

function M:getInfo(key)
	if key then
		return self.info[key]
	else
		return self.info
	end
end

return M