--[[

文件初始化

]]


require("GameLuaScript/Config/channel")


local initCopyLayer = {
	layer,
	viewLayer,
	tipsLayer
}

-- 接口
local interface_java = UpdataRes:getInstance()		-- android
local interface_cs = UpdataRes:getInstance()		-- wp8
local platform_type = interface_java:get_type()
local interface_oc
if platform_type == 2 then 				-- ios
	interface_oc = UpdateDataOC:getInstance()
end

local update_url = ""

function initCopyLayer:new(params)
	local this = {}
	setmetatable(this,self)
	self.__index  = self

	params = params or {}

	this.layer = CCLayer:create()

	local bg = CCSprite:create("image/scene/login/bg.jpg")
	INIT_FUNCTION.setAnchPos( bg , 0 , 0 )
	this.layer:addChild( bg )

	if CHANNEL_GROUP == "tgxy" or CHANNEL_GROUP == "tgxy2" then
		local temp = CCSprite:create("image/scene/login/role.png")
		INIT_FUNCTION.setAnchPos( temp , INIT_FUNCTION.cx , 536 , 0.5 , 0.5 )
		this.layer:addChild( temp )

		local temp = CCSprite:create("image/scene/login/logo.png")
		INIT_FUNCTION.setAnchPos( temp , INIT_FUNCTION.cx , 321 , 0.5 , 0.5 )
		this.layer:addChild( temp )
	end

	self:init()

	if WRITE_PATH == "" then
		-- 直接进入游戏
		self:go()
	else
		-- 验证资源
		this:checkUniq()
	end
	return this
end

function initCopyLayer:getLayer()
	return self.layer
end





-- 验证安装包是否一致，不一致则删除文件
function initCopyLayer:checkUniq()
	if self.viewLayer then
		self.viewLayer:removeFromParentAndCleanup(true)
		self.viewLayer = nil
	end

	if platform_type == 2 or platform_type == 3 then
		-- IOS 直接走copyfile, 删文件和拷文件是一起的 -- WP8 same as IOS
		self:copyfiles()
		return
	end

	local check_uniq = CCFileUtils:sharedFileUtils():android_checkUniq(WRITE_PATH)

	-- 判断是否需要删缓存的文件(version文件不为空，并且uniq一致)
	CCLuaLog("check uniq: " .. (check_uniq and "true" or "false"))
	if check_uniq == false then
		self:checkUpdate()
		return
	end

	-- 需要删文件
	self.viewLayer = CCLayer:create()

	local loading_box = CCSprite:create("image/box.png")
	INIT_FUNCTION.setAnchPos( loading_box , INIT_FUNCTION.cx , 50 , 0.5 , 0 )
	self.viewLayer:addChild( loading_box )
	
	local loading = CCSprite:create("image/loading.png")
	INIT_FUNCTION.setAnchPos( loading , INIT_FUNCTION.cx - 120 , 102 , 0.5 , 0.5 )
	self.viewLayer:addChild( loading )

	local action = CCRepeatForever:create( CCRotateBy:create(0.5 , 180) )
	loading:runAction(action)

	local label = CCLabelTTF:create("正在删除老版本...\n可能需要30秒~1分钟" , "Thonburi" , 20)
	INIT_FUNCTION.setAnchPos( label , INIT_FUNCTION.cx - 65 , 75 )
	label:setHorizontalAlignment(0)
	label:setColor( ccc3( 0xff , 0xff , 0xff ) )
	self.viewLayer:addChild(label )

	-- 开始删除文件(需要过一定时间，让主进程可以渲染页面)
	local handle
	handle = CCDirector:sharedDirector():getScheduler():scheduleScriptFunc(function()
		CCDirector:sharedDirector():getScheduler():unscheduleScriptEntry(handle)
		handle = nil

		-- 调用本地接口
		CCLuaLog("# Begin Delete Files...")
		interface_java:deleData(WRITE_PATH , "true")
		CCLuaLog("# Finish Delete Files...")

		-- 删完以后，开始拷贝文件
		self:copyfiles()
	end , 0.01 , false)

	self.layer:addChild(self.viewLayer)
end


-- 拷贝文件到SD卡
function initCopyLayer:copyfiles()
	CCLuaLog("# Check Copy Files...")
	if self.viewLayer then
		self.viewLayer:removeFromParentAndCleanup(true)
		self.viewLayer = nil
	end


	-- 需要拷贝文件
	self.viewLayer = CCLayer:create()

	local loading_box = CCSprite:create("image/box.png")
	INIT_FUNCTION.setAnchPos( loading_box , INIT_FUNCTION.cx , 50 , 0.5 , 0 )
	self.viewLayer:addChild( loading_box )
--	
--	local loading = CCSprite:create("image/loading.png")
--	INIT_FUNCTION.setAnchPos( loading , INIT_FUNCTION.cx - 120 , 102 , 0.5 , 0.5 )
--	self.viewLayer:addChild( loading )
--
--	local action = CCRepeatForever:create( CCRotateBy:create(0.5 , 180) )
--	loading:runAction(action)

	local label = CCLabelTTF:create("正在解压资源,请稍候...可能需要5-30秒\n资源解压过程不消耗网络流量" , "Thonburi" , 16)
	INIT_FUNCTION.setAnchPos( label , INIT_FUNCTION.cx , 55 , 0.5 )
	label:setHorizontalAlignment(0)
	label:setColor( ccc3( 0xff , 0xff , 0xff ) )
	self.viewLayer:addChild(label )
	
	refreshCopy(0)
--
--	-- 开始部署文件(需要过一定时间，让主进程可以渲染页面)
	CCLuaLog("# Start Copy Files...")
	if platform_type == 1 or platform_type == 3 then -- android + wp8
		local handle
		handle = CCDirector:sharedDirector():getScheduler():scheduleScriptFunc(function()
			CCDirector:sharedDirector():getScheduler():unscheduleScriptEntry(handle)
			handle = nil
			if platform_type == 1 then
				interface_java:copydata("" , WRITE_PATH , function(_ , _)end) 
			elseif platform_type == 3 then
				interface_cs:copydata("" , WRITE_PATH , function(_ , _)end) 
			end
		end , 0.05 , false)
	elseif platform_type == 2 then
		local handle
		handle = CCDirector:sharedDirector():getScheduler():scheduleScriptFunc(function()
			CCDirector:sharedDirector():getScheduler():unscheduleScriptEntry(handle)
			handle = nil
			
			interface_oc:checkResources()
		end , 0.05 , false)
	end

	self.layer:addChild(self.viewLayer)



	--[[上报统计]]
	xpcall(function()
		local deviceinfo = nil
		if platform_type == 1 then 			-- android
			deviceinfo = interface_java:DeviceId() or "unknown"
		elseif platform_type == 2 then    -- ios
			deviceinfo = interface_oc:deviceId() or "unknown"
		end

		if deviceinfo ~= nil and type(deviceinfo) == "string" then
			require("GameLuaScript/Config/base")
			INIT_FUNCTION:httpPost(CONFIG_STAT_URL , {
				action = "active",
				info = deviceinfo,
				channel = CHANNEL_ID
			} , function(_ , _)end)
		end
	end , function() end)
end

-- 检查在线版本更新
function initCopyLayer:checkUpdate()
	CCLuaLog("# Check Update...")
	if self.viewLayer then
		self.viewLayer:removeFromParentAndCleanup(true)
		self.viewLayer = nil
	end

	self.viewLayer = CCLayer:create()

	local loading_box = CCSprite:create("image/box.png")
	INIT_FUNCTION.setAnchPos( loading_box , INIT_FUNCTION.cx , 50 , 0.5 , 0 )
	self.viewLayer:addChild( loading_box )
	
	local loading = CCSprite:create("image/loading.png")
	INIT_FUNCTION.setAnchPos( loading , INIT_FUNCTION.cx - 120 , 102 , 0.5 , 0.5 )
	self.viewLayer:addChild( loading )

	local action = CCRepeatForever:create( CCRotateBy:create(0.5 , 180) )
	loading:runAction(action)

	local label = CCLabelTTF:create("正在检查新资源,请稍候..." , "Thonburi" , 20)
	INIT_FUNCTION.setAnchPos( label , INIT_FUNCTION.cx - 65 , 85 )
	label:setHorizontalAlignment(0)
	label:setColor( ccc3( 0xff , 0xff , 0xff ) )
	self.viewLayer:addChild(label )

	self.layer:addChild(self.viewLayer)

	-- 开始发包检查新版本
	require "GameLuaScript/Config/base"
	update_url = CONFIG_UPDATA_URL
	
	local url = update_url .. "/versionup.php"
	CCLuaLog("# Update URL: " .. url)

	-- 获取当前版本号
	--[[
	require "GameLuaScript/Config/version"
	local version = VERSION
	]]
	local version = ""
	local version_content = INIT_FUNCTION.readfile(WRITE_PATH .. "GameLuaScript/Config/version.lua")
	if version_content ~= nil then
		local _ , _ , temp = string.find(version_content , 'VERSION = "(%d%d%.%d%d%.%d%d%.%d)"')
		version = temp
	else
		require "GameLuaScript/Config/version"
		version = VERSION
	end

	CCLuaLog("# Current Version: " .. (version or "nil"))

	if version == nil then
		CCLuaLog("# No Version Update")
		self:go()
		return
	end

	local version_num = string.gsub(version , "%." , "")
    
	local isOver = 0
	local isExsitRes = 0
	INIT_FUNCTION:httpPost(url , {channel = CHANNEL_ID , channel_group = CHANNEL_GROUP , version = version_num , jiami = __INNER_JIAMI__ ,isOver30=isOver, isExsitRes30=isExsitRes } , function(http_code , response)
		CCLuaLog("# Response back")

		if type(response) ~= "string" or response == "" then
			CCLuaLog("# Get Version Error.")
			response = "version=0"

			ERROR_CODE = -1
			ERROR_MSG = "网络异常，检查新版本失败"
		end
		print(response)
		local rows = INIT_FUNCTION.split(response , "\n")
		local version_data = INIT_FUNCTION.split(rows[1] , "=")
		if #version_data ~= 2 then
			CCLuaLog("# Get Version Error..")

			ERROR_CODE = http_code
			ERROR_MSG = "网络异常，检查新版本失败\n" .. response

			self:go()
			return
		end
		
		CCLuaLog("# New Version: " .. version_data[2])

		if version_data[2] == "0" then
			self:go()
			return
		end

		-- 显示升级界面
		self:newVersion(version , version_data[2] , rows)
	end)
end


function initCopyLayer:newVersion(old_version , new_version , rows)
	if self.viewLayer then
		self.viewLayer:removeFromParentAndCleanup(true)
		self.viewLayer = nil
	end

	self.viewLayer = CCLayer:create()

	local version_label = CCLabelTTF:create(old_version , "Thonburi" , 18)
	version_label:setColor( ccc3( 0x4d , 0x15 , 0x15 ) )
	INIT_FUNCTION.setAnchPos(version_label , 10 , 10)
	self.viewLayer:addChild( version_label )

	local label = CCLabelTTF:create( "发现新版本 " .. "V" ..new_version , "Thonburi" , 36)
	label:setColor(ccc3( 0x2c , 0x00 , 0x00 ))
	INIT_FUNCTION.setAnchPos(label , INIT_FUNCTION.cx , 180 , 0.5)
	self.viewLayer:addChild(label)
	
    local bg = CCSprite:create("image/scene/updata/bar_1.png")
    INIT_FUNCTION.setAnchPos(bg , 67 , 136 , 0 , 0.5)
    self.viewLayer:addChild(bg)

    local bg1 = CCSprite:create("image/scene/updata/bar_0.png")
	INIT_FUNCTION.setAnchPos(bg1 , 67 , 136 , 0 , 0.5)
	self.viewLayer:addChild(bg1)
	bg1:setTextureRect(CCRectMake(0,0,1,39))
	
	local updata_font_max = CCLabelTTF:create( "" , "Thonburi" , 20 )
	INIT_FUNCTION.setAnchPos(updata_font_max , 410 , 93 , 1 , 0)
	updata_font_max:setColor(ccc3( 0x2c , 0x00 , 0x00 ))
	self.viewLayer:addChild(updata_font_max)

	local update_filename = CCLabelTTF:create( "" , "Thonburi" , 20 )
	INIT_FUNCTION.setAnchPos(update_filename , 70 , 93)
	update_filename:setColor(ccc3( 0x2c , 0x00 , 0x00 ))
	self.viewLayer:addChild(update_filename)
	
	-- 按钮
	local btn_callback = nil
	local btn_bg = CCSprite:create("image/scene/updata/button.png")
	INIT_FUNCTION.setAnchPos(btn_bg , 167 , 45)
	self.viewLayer:addChild(btn_bg)
	local btn_rect = CCRectMake( (platform_type ~= 3 and btn_bg.x or btn_bg:getPositionX()), (platform_type ~= 3 and btn_bg.y or btn_bg:getPositionY()) , btn_bg:getContentSize().width , btn_bg:getContentSize().height)

	local btn_grey_bg = CCSprite:create("image/scene/updata/button_grey.png")
	INIT_FUNCTION.setAnchPos(btn_grey_bg , 167 , 45)
	self.viewLayer:addChild(btn_grey_bg)
	btn_grey_bg:setVisible(false)
	
	local font_img = CCSprite:create("image/scene/updata/font.png")
	INIT_FUNCTION.setAnchPos(font_img , 180 , 42)
	self.viewLayer:addChild(font_img)

	self.layer:addChild(self.viewLayer)

	-- 按钮是否正在执行
	local btn_progress = false

	-- 计算总大小
	local filesize_max = 0
	for i = 2 , #rows do
		local temp = INIT_FUNCTION.split(rows[i] , "\t")
		if #temp >= 2 then
			local download_size = (temp[3] and tonumber(temp[3]) > 0) and temp[3] or temp[2]
			filesize_max = filesize_max + tonumber(download_size) / 1024
		end
	end
	filesize_max = string.format("%.2f" , filesize_max)
	filesize_max = tonumber(filesize_max)
	updata_font_max:setString("0/" .. filesize_max .. "K")


	-- 更新进度条
	local filesize_cur = 0
	local function updateBar(cur)
		cur = string.format("%.2f" , cur)
		cur = tonumber(cur)

		if type(cur) ~= "number" then return end

		if cur >= filesize_max then
			cur = filesize_max
		end
		
		updata_font_max:setString(cur .. "/" .. filesize_max .. "K")
		
		
		local width = math.floor( cur * (346 / (filesize_max)) )		

		if width > 346 then width = 346 end
		if width <= 0 or width~=width then width = 1 end
		bg1:setTextureRect( CCRectMake(0 , 0 , width , 39) )
	end

	--[[ 更新文件 ]]
	local function updateOneFile(url , path , name , filesize , download_size , callback)
        if WRITE_PATH == "" then
            print("哥,WRITE_PATH不对,是空的")
            callback(0)
            return
        end

        -- 兼容&容错
        if not download_size then download_size = filesize end

        local show_name = string.gsub(name , "%.lua" , "")
        update_filename:setString( show_name )

		INIT_FUNCTION:httpGet(url , function(http_code , response)
			-- 网络请求超时
			CCLuaLog("========================")
			CCLuaLog(string.len(response))
			CCLuaLog(tonumber(filesize))
			if http_code == -28 or type(response) ~= "string" or string.len(response) ~= tonumber(filesize) then
				if http_code == -28 then
					self:showTips("下载文件超时\n请点击按钮重试")
				else
					self:showTips("下载文件异常\n请点击按钮重试")
				end

				btn_grey_bg:setVisible(false)
				btn_bg:setVisible(true)
				btn_progress = false

				btn_callback = function()
					self:hideTips()

					-- 重新开始下载
					updateOneFile(url , path , name , filesize , download_size , callback)
				end

				return
			end

			-- 有一些特殊文件，不更新
			if name == "uniq.lua" or name == "channel.lua" then
				callback(0)
				return
			end

            local full_path = WRITE_PATH .. path .. name

            local temp = INIT_FUNCTION.split(path , "/")
            local dir = ""
            for i = 1 , #temp do
                if temp[i] ~= "" then
                    dir = dir .. temp[i] .. "/"

                    local temp_fp = io.open(WRITE_PATH .. dir , "r")
                    if temp_fp then
                        io.close(temp_fp)
                    else
                        if platform_type == 2 then
                              UpdateDataOC:getInstance():createDir(WRITE_PATH .. dir)
                        elseif platform_type == 3 then
                        	  interface_cs:createDir(WRITE_PATH .. dir)
                        else
                              os.execute("mkdir \"" .. WRITE_PATH .. dir .. "\"")
                        end
                    end
                end
            end
        
    	    local fp = io.open(full_path , "w+b")
            if fp then
                fp:write(response)
                fp:close()
            end

			callback(http_code)
		end , {
			timeout = math.max( math.ceil(filesize / 1024) , 15),
			progress_callback = function(total , now)
				local rate = now * (download_size / filesize)
				updateBar(filesize_cur + (rate / 1024) )
			end
		})
	end

	--[[ 更新文件 ]]
	local function beginUpdateFile()
		local index = 2 	-- 从第二个文件开始
		local version_path
		local version_url
		local version_file_size

		local function go()
			if type(rows[index]) == "string" and rows[index] ~= "" then
				local url_data = INIT_FUNCTION.split(rows[index] , "\t")

				if #url_data < 2 then
					return false
				end

				local full_url = url_data[1]
				local temp = INIT_FUNCTION.split(full_url , '/')
				local path = ""
				local name = temp[#temp]
				for i = 4 , #temp - 1 do
					path = path .. temp[i] .. "/"
				end


				if name == "version.lua" then
					version_path = path
					version_url = full_url
					version_file_size = url_data[2]

					-- 继续执行下一个
					index = index + 1
					go()
					return true
				end

				updateOneFile(update_url .. "/" .. full_url , path , name , url_data[2] , url_data[3] , function(http_code)
					if http_code ~= 0 then
						return
					end

					-- 更新界面
					local download_size = url_data[3] and url_data[3] or url_data[2]
					filesize_cur = filesize_cur + download_size / 1024
					updateBar(filesize_cur)

					-- 继续执行下一个
					index = index + 1
					go()
				end)
			else
				-- 开始更新版本号
				if version_path and version_url and version_file_size then
					updateOneFile(update_url .. "/" .. version_url , version_path , "version.lua" , version_file_size , version_file_size , function(http_code)
						self.go()
					end)
				end
			end
		end

		-- 开始
		go()
	end
	
	


	-- 按钮点击事件
	btn_callback = function()
		beginUpdateFile()
	end
	
	local function onTouch(eventType , x , y)
		if eventType == CCTOUCHBEGAN then
			if btn_progress == false then
				if btn_rect:containsPoint( ccp(x , y) ) then
					btn_progress = true
					btn_grey_bg:setVisible(true)
					btn_bg:setVisible(false)

					btn_callback()
				end
			end
			return true
		end
		return true
	end
	self.viewLayer:registerScriptTouchHandler(onTouch)
    self.viewLayer:setTouchEnabled(true)
end


-- 进入主界面
function initCopyLayer:go()
	-- 添加搜索路径
	if platform_type ~= 3 then
		CCFileUtils:sharedFileUtils():setResourceDirectory(WRITE_PATH)
	end

	-- --模拟器需求
	-- if kTargetMacOS == CCApplication:sharedApplication():getTargetPlatform() then
	-- 	if CHANNEL_GROUP == "tmsj" then
	-- 		CCFileUtils:sharedFileUtils():setResourceDirectory("/workspace/cocos2d-2.0-x-2.0.4/GameLua/version_tmsj")
	-- 	elseif CHANNEL_GROUP == "hy" then
	-- 		CCFileUtils:sharedFileUtils():setResourceDirectory("/workspace/cocos2d-2.0-x-2.0.4/GameLua/version_hysj")
	-- 	elseif CHANNEL_GROUP == "td" then
	-- 		CCFileUtils:sharedFileUtils():setResourceDirectory("/workspace/cocos2d-2.0-x-2.0.4/GameLua/version_td")
	-- 	elseif CHANNEL_GROUP  == "tgxy" then
	-- 		CCFileUtils:sharedFileUtils():setResourceDirectory("/workspace/cocos2d-2.0-x-2.0.4/GameLua/version_tgxy")
	-- 	elseif CHANNEL_GROUP  == "tgxy2" then
	-- 		CCFileUtils:sharedFileUtils():setResourceDirectory("/workspace/cocos2d-2.0-x-2.0.4/GameLua/version_tgxy")
	-- 		CCFileUtils:sharedFileUtils():setResourceDirectory2("/workspace/cocos2d-2.0-x-2.0.4/GameLua/version_tgxy2")
	-- 	end
	-- elseif platform_type ~= 3 then
	-- 	CCFileUtils:sharedFileUtils():setResourceDirectory(WRITE_PATH)
	-- end


	CCLuaLog("# All is ok")

	-- 所有包都加密
	if platform_type ~= 0 and __INNER_JIAMI__ == 1 then
		local status, msg = pcall(function()
			dofile(WRITE_PATH .. "GameLuaScript/fileFilter.lua")
		end)
		if not status then
			CCLuaLog(msg)
			require("GameLuaScript/fileFilter")
		end
		
		
		CCLuaLog("==========> require decode <=========="); KNLuaLoader.change_lua_loader()
	end

	local handle
	handle = CCDirector:sharedDirector():getScheduler():scheduleScriptFunc(function()
		CCDirector:sharedDirector():getScheduler():unscheduleScriptEntry(handle)
		handle = nil

		requires(IMG_PATH , "GameLuaScript/GameCanvasLua")
	end , 0.01 , false)
	
end



-- 初始化
function initCopyLayer:init()
	--[[	
	if CHANNEL_ID == "uc" then
		UCGameSDK:getInstance():setLogLevel(2)
		UCGameSDK:getInstance():setOrientation(0)
		UCGameSDK:getInstance():initSDK(false , 2 , 27820 , 518084 , 2126 , "安卓大话水浒" , true , true)
	end
	]]
end


-- show tips
function initCopyLayer:showTips(msg)
	if self.tipsLayer then
		self.tipsLayer:removeFromParentAndCleanup(true)
		self.tipsLayer = nil
	end

	self.tipsLayer = CCSprite:create("image/common/prompt_bg.png")
	INIT_FUNCTION.setAnchPos(self.tipsLayer , INIT_FUNCTION.cx , INIT_FUNCTION.cy , 0.5)

	local textField = CCLabelTTF:create(msg , "Thonburi" , 20)
	textField:setColor(ccc3( 0xff , 0xff , 0xff ))
	textField:setAnchorPoint( ccp( 0 , 1 ) )

	if textField:getContentSize().width > 360 then
		textField:setDimensions(CCSize:new( 360 , 70 ))
		textField:setPosition( ccp( ( self.tipsLayer:getContentSize().width - textField:getContentSize().width ) / 2 , self.tipsLayer:getContentSize().height / 1.5 + 10 ) )
	else
		textField:setPosition( ccp( ( self.tipsLayer:getContentSize().width - textField:getContentSize().width ) / 2 , self.tipsLayer:getContentSize().height / 1.5 - 8 ) )
	end

	self.tipsLayer:addChild(textField)

	self.viewLayer:addChild(self.tipsLayer)
end

function initCopyLayer:hideTips()
	if self.tipsLayer then
		self.tipsLayer:removeFromParentAndCleanup(true)
		self.tipsLayer = nil
	end
end

return initCopyLayer
